package jp.veritrans.mpos.sample.integration.ui;

import android.app.Activity;
import android.os.Bundle;
import android.view.View;
import android.widget.CompoundButton;
import android.widget.EditText;
import android.widget.TextView;
import android.widget.Toast;

import androidx.annotation.NonNull;
import androidx.fragment.app.Fragment;

import com.google.android.material.switchmaterial.SwitchMaterial;

import java.util.Objects;

import jp.veritrans.mpos.sample.integration.R;
import jp.veritrans.mpos.v2.integration.MposIntegrationAction;
import jp.veritrans.mpos.v2.integration.MposIntegrationService;
import jp.veritrans.mpos.v2.integration.OnNotStartActivityListener;

/**
 * mPOSアプリと連携するため為の基底となるフラグメント。
 */
public abstract class IntegrationBaseFragment extends Fragment
        implements CompoundButton.OnCheckedChangeListener, OnNotStartActivityListener {

    @Override
    public abstract void onCheckedChanged(CompoundButton buttonView, boolean isChecked);

    @Override
    public void onNotStartActivity() {

        Activity activity = getActivity();
        if (activity != null) {
            Toast.makeText(activity, getString(R.string.msg_mpos_app_not_found), Toast.LENGTH_SHORT).show();
        }
    }

    /**
     * エディットテキストとスイッチの状態を指定した{@link Bundle}に保存する。
     *
     * @param outState 保存先となる{@link Bundle}
     * @param combs {@link EditTextAndSwitchCombination}の配列
     */
    void saveInstanceState(Bundle outState, EditTextAndSwitchCombination[] combs) {

        View view = getView();
        if (view == null) {
            return;
        }

        for (EditTextAndSwitchCombination comb : combs) {
            SwitchMaterial sw = view.findViewById(comb.getSwitchId());
            EditText txt = view.findViewById(comb.getEditTextId());
            outState.putBoolean(Integer.toString(comb.getSwitchId()), sw.isChecked());
            outState.putString(Integer.toString(comb.getEditTextId()), txt.getText().toString());
        }
    }

    /**
     * {@link Bundle}からエディットテキストとスイッチの状態を復元する。
     *
     * @param v {@link View}
     * @param savedInstanceState 復元のためのデータが格納された{@link Bundle}
     * @param combs {@link EditTextAndSwitchCombination}の配列
     */
    void restoreInstanceState(View v, Bundle savedInstanceState, EditTextAndSwitchCombination[] combs) {

        if (v == null) {
            return;
        }

        for (EditTextAndSwitchCombination comb : combs) {
            SwitchMaterial sw = v.findViewById(comb.getSwitchId());
            EditText txt = v.findViewById(comb.getEditTextId());

            if (savedInstanceState != null) {
                sw.setChecked(savedInstanceState.getBoolean(Integer.toString(comb.getSwitchId())));
                txt.setText(savedInstanceState.getString(Integer.toString(comb.getEditTextId())));
            }
            setTextViewEnabled(v, comb.getEditTextId(), sw.isChecked());
            sw.setOnCheckedChangeListener(this);
        }
    }

    /**
     * 指定した{@link TextView}を有効化・無効化する。
     *
     * @param v {@link View}
     * @param targetId {@link TextView}のID
     * @param enabled 有効化する場合は{@code true}を指定する。<br>
     *                 無効化する場合は{@code false}を指定する。
     */
    void setTextViewEnabled(View v, int targetId, boolean enabled) {

        TextView text = v.findViewById(targetId);
        if (text != null) {
            text.setEnabled(enabled);
            text.setFocusable(enabled);
            text.setFocusableInTouchMode(enabled);
        }
    }

    /**
     * {@link MposIntegrationService}のインスタンスを生成する。
     *
     * @param action アクション
     * @return {@link MposIntegrationService}のインスタンス
     */
    MposIntegrationService newMposIntegrationService(@NonNull MposIntegrationAction action) {

        String packageName = Objects.toString(getString(R.string.mpos_package_name), "");
        if (packageName.isEmpty()) {
            return new MposIntegrationService(action);
        }
        return new MposIntegrationService(action, packageName);
    }

    /**
     * mPOSアプリに連携する拡張データを生成する。
     *
     * @return 拡張データ
     */
    Bundle createMposIntegrationExtendedData() {
        return new Bundle();
    }

    /**
     * {@link EditTextAndSwitchCombination}が指すエディットテキストに設定された値を
     * 指定した{@link Bundle}に挿入する。<br>
     * {@link EditTextAndSwitchCombination}が指すスイッチがOFF担っている場合は、
     * {@link Bundle}への値の挿入は行わない。
     *
     * @param bundle 挿入先となる{@link Bundle}
     * @param bundleKey {@link Bundle}に挿入する際のキー
     * @param comb 挿入元が定義された{@link EditTextAndSwitchCombination}
     */
    void putBundleData(Bundle bundle, String bundleKey, EditTextAndSwitchCombination comb) {

        View view = getView();
        if (view == null) {
            return;
        }

        SwitchMaterial sw = view.findViewById(comb.getSwitchId());
        if (sw == null || sw.isChecked()) {
            EditText edit = view.findViewById(comb.getEditTextId());
            String value = edit == null ? null : edit.getText().toString();
            bundle.putString(bundleKey, value);
        }
    }

    /**
     * エディットテキストとスイッチの組み合わせを定義するインターフェース。
     */
    interface EditTextAndSwitchCombination {

        /**
         * エディットテキストのIDを取得する。
         *
         * @return エディットテキストのID
         */
        int getEditTextId();

        /**
         * スイッチのIDを取得する。
         *
         * @return スイッチのID
         */
        int getSwitchId();
    }
}
