package jp.veritrans.mpos.v2.integration;

import android.app.Activity;
import android.content.Intent;
import android.content.pm.PackageManager;
import android.os.Bundle;

/**
 * mPOSアプリとの連携を行うクラス。
 */
@SuppressWarnings({"WeakerAccess", "unused"})
public class MposIntegrationService {

    /** mPOSアプリパッケージ名 */
    private static final String MPOS_PACKAGE_NAME = "jp.veritrans.mpos.v2.app.mpospay";

    /** mPOSアプリ連携先クラス名 */
    private static final String MPOS_INTEGRATION_CLASS_NAME = "jp.veritrans.mpos.v2.app.scene.BootScene";
    /** 処理結果ステータス：成功 */
    private static final String STATUS_SUCCESS = "success";
    /** 処理結果ステータス：失敗 */
    private static final String STATUS_FAILURE = "failure";
    /** 処理結果ステータス：中止 */
    private static final String STATUS_CANCEL = "cancel";

    /** 連携先パッケージ */
    private final String targetPackage;
    /** アクション */
    private final MposIntegrationAction action;
    /** コールバックパッケージ名 */
    private String callbackPackageName;
    /** コールバック成功クラス名 */
    private String callbackSuccessClassName;
    /** コールバック失敗クラス名 */
    private String callbackErrorClassName;
    /** コールバック中止クラス名 */
    private String callbackCancelClassName;
    /** コールバック成功起動フラグ */
    private int callbackSuccessActivityFlag = Intent.FLAG_ACTIVITY_NEW_TASK;
    /** コールバック失敗起動フラグ */
    private int callbackErrorActivityFlag = Intent.FLAG_ACTIVITY_NEW_TASK;
    /** コールバック中止起動フラグ */
    private int callbackCancelActivityFlag = Intent.FLAG_ACTIVITY_NEW_TASK;
    /** mPOSアプリに連携できないことを通知するリスナー */
    private OnNotStartActivityListener onNotStartActivityListener;

    /**
     * コンストラクタ。
     *
     * @param action アクション
     */
    public MposIntegrationService(MposIntegrationAction action) {
        this(action, MPOS_PACKAGE_NAME);
    }

    /**
     * コンストラクタ。
     *
     * @param action アクション
     * @param target 連携先パッケージ名
     */
    public MposIntegrationService(MposIntegrationAction action, String target) {

        if (action == null) {
            throw new NullPointerException("Args: Action is null.");
        }

        if (target == null) {
            throw new NullPointerException("Args: Target is null.");

        } else if (target.trim().isEmpty()) {
            throw new IllegalArgumentException("Args: Target is blank.");
        }
        this.action = action;
        this.targetPackage = target;
    }

    /**
     * コールバックパッケージ名を設定する。
     *
     * @param callbackPackageName コールバックパッケージ名
     */
    public void setCallbackPackageName(String callbackPackageName) {
        this.callbackPackageName = callbackPackageName;
    }

    /**
     * mPOSアプリに連携した結果、処理が成功した場合の戻り先となるアクティビティクラスを設定する。
     *
     * @param callbackSuccessClass 処理成功時の戻り先アクティビティクラス
     */
    public void setCallbackSuccessClassName(Class<? extends Activity> callbackSuccessClass) {
        this.callbackSuccessClassName = callbackSuccessClass.getName();
    }

    /**
     * mPOSアプリに連携した結果、処理が失敗した場合の戻り先となるアクティビティクラスを設定する。
     *
     * @param callbackErrorClass 処理失敗時の戻り先アクティビティクラス
     */
    public void setCallbackErrorClassName(Class<? extends Activity> callbackErrorClass) {
        this.callbackErrorClassName = callbackErrorClass.getName();
    }

    /**
     * mPOSアプリに連携した結果、処理を中止した場合の戻り先となるアクティビティクラスを設定する。
     *
     * @param callbackCancelClassName 処理中止時の戻り先アクティビティクラス
     */
    public void setCallbackCancelClassName(Class<? extends Activity> callbackCancelClassName) {
        this.callbackCancelClassName = callbackCancelClassName.getName();
    }

    /**
     * mPOSアプリからお客様作成のアプリへ処理成功として戻る際のアクティビティの起動方法を追加する。
     *
     * @param callbackSuccessActivityFlag 起動フラグ
     */
    public void addCallbackSuccessActivityFlag(int callbackSuccessActivityFlag) {
        this.callbackSuccessActivityFlag |= callbackSuccessActivityFlag;
    }

    /**
     * mPOSアプリからお客様作成のアプリへ処理成功として戻る際のアクティビティの起動方法を設定する。
     *
     * @param callbackSuccessActivityFlag 起動フラグ
     */
    public void setCallbackSuccessActivityFlag(int callbackSuccessActivityFlag) {
        this.callbackSuccessActivityFlag = callbackSuccessActivityFlag;
    }

    /**
     * mPOSアプリからお客様作成のアプリへ処理失敗として戻る際のアクティビティの起動方法を追加する。
     *
     * @param callbackErrorActivityFlag 起動フラグ
     */
    public void addCallbackErrorActivityFlag(int callbackErrorActivityFlag) {
        this.callbackErrorActivityFlag |= callbackErrorActivityFlag;
    }

    /**
     * mPOSアプリからお客様作成のアプリへ処理失敗として戻る際のアクティビティの起動方法を設定する。
     *
     * @param callbackErrorActivityFlag 起動フラグ
     */
    public void setCallbackErrorActivityFlag(int callbackErrorActivityFlag) {
        this.callbackErrorActivityFlag = callbackErrorActivityFlag;
    }

    /**
     * mPOSアプリからお客様作成のアプリへ処理中止として戻る際のアクティビティの起動方法を追加する。
     *
     * @param callbackCancelActivityFlag 起動フラグ
     */
    public void addCallbackCancelActivityFlag(int callbackCancelActivityFlag) {
        this.callbackCancelActivityFlag |= callbackCancelActivityFlag;
    }

    /**
     * mPOSアプリからお客様作成のアプリへ処理中止として戻る際のアクティビティの起動方法を設定する。
     *
     * @param callbackCancelActivityFlag 起動フラグ
     */
    public void setCallbackCancelActivityFlag(int callbackCancelActivityFlag) {
        this.callbackCancelActivityFlag = callbackCancelActivityFlag;
    }

    /**
     * mPOSアプリがインストールされていないなど、mPOSアプリに連携できない場合に
     * コールバックされるリスナーを設定する。
     *
     * @param onNotStartActivityListener {@link OnNotStartActivityListener}
     */
    public void setOnNotStartActivityListener(OnNotStartActivityListener onNotStartActivityListener) {
        this.onNotStartActivityListener = onNotStartActivityListener;
    }

    /**
     * mPOSアプリを起動する。
     *
     * @param activity {@link Activity}
     * @param extras mPOSアプリに連携する拡張（処理固有）データが設定された{@link Bundle}
     * @return 連携した{@link Intent}
     */
    public Intent startMposApplication(Activity activity, Bundle extras) {

        Intent intent = newMposIntegrationIntent(activity);
        if (intent != null) {
            if (extras != null) {
                intent.putExtras(extras);
            }

            intent.putExtra(MposIntegrationKey.ACTION, action.getType());
            intent.putExtra(MposIntegrationKey.CALLBACK_PACKAGE_NAME, callbackPackageName);
            intent.putExtra(MposIntegrationKey.CALLBACK_SUCCESS_CLASS_NAME, callbackSuccessClassName);
            intent.putExtra(MposIntegrationKey.CALLBACK_ERROR_CLASS_NAME, callbackErrorClassName);
            intent.putExtra(MposIntegrationKey.CALLBACK_CANCEL_CLASS_NAME, callbackCancelClassName);
            intent.putExtra(MposIntegrationKey.CALLBACK_SUCCESS_ACTIVITY_FLAG, callbackSuccessActivityFlag);
            intent.putExtra(MposIntegrationKey.CALLBACK_ERROR_ACTIVITY_FLAG, callbackErrorActivityFlag);
            intent.putExtra(MposIntegrationKey.CALLBACK_CANCEL_ACTIVITY_FLAG, callbackCancelActivityFlag);

            activity.startActivity(intent);

        } else if (onNotStartActivityListener != null) {
            onNotStartActivityListener.onNotStartActivity();
        }
        return intent;
    }

    /**
     * mPOSアプリに連携するための{@link Intent}を生成する。
     *
     * @param activity {@link Activity}
     * @return {@link Intent}
     */
    private Intent newMposIntegrationIntent(Activity activity) {

        PackageManager pm = activity.getPackageManager();
        Intent intent = pm.getLaunchIntentForPackage(targetPackage);
        if (intent != null) {
            intent.setClassName(targetPackage, MPOS_INTEGRATION_CLASS_NAME);
            intent.setFlags(Intent.FLAG_ACTIVITY_NEW_TASK | Intent.FLAG_ACTIVITY_CLEAR_TASK);
        }
        return intent;
    }

    /**
     * 指定したアクティビティがmPOSアプリから呼び出されたかを判定する。
     *
     * @param activity {@link Activity}
     * @return mPOSアプリから呼び出された場合は{@code true}を取得する。<br>
     *          mPOSアプリから呼び出されていない場合は{@code false}を取得する。
     */
    public static boolean isCallingMposActivity(Activity activity) {
        return isCallingMposActivity(activity, MPOS_PACKAGE_NAME);
    }
    /**
     * 指定したアクティビティが指定したパッケージから呼び出されたかを判定する。
     *
     * @param activity {@link Activity}
     * @param target パッケージ名
     * @return 指定したパッケージから呼び出された場合は{@code true}を取得する。<br>
     *          指定したパッケージから呼び出されていない場合は{@code false}を取得する。
     */
    public static boolean isCallingMposActivity(Activity activity, String target) {

        if (activity == null || target == null || target.trim().isEmpty()) {
            return  false;
        }

        boolean result = false;
        Intent intent = activity.getIntent();
        if (intent != null) {
            String packageName = intent.getStringExtra(MposIntegrationKey.CALLING_PACKAGE_NAME);
            result = packageName != null && packageName.startsWith(target);

            // 下位バージョンとの互換のために成功・失敗・中止時の遷移先クラス名項目の有無で判定する。
            if (!result) {
                packageName = intent.getStringExtra(MposIntegrationKey.CALLBACK_SUCCESS_CLASS_NAME);
                result = packageName != null && !packageName.isEmpty();
            }

            if (!result) {
                packageName = intent.getStringExtra(MposIntegrationKey.CALLBACK_ERROR_CLASS_NAME);
                result = packageName != null && !packageName.isEmpty();
            }

            if (!result) {
                packageName = intent.getStringExtra(MposIntegrationKey.CALLBACK_CANCEL_CLASS_NAME);
                result = packageName != null && !packageName.isEmpty();
            }
        }
        return result;
    }

    /**
     * mPOSアプリからレスポンスとして連携された処理結果が成功を示すものであるかを判定する。
     *
     * @param intent {@link Intent}
     * @return 処理結果が成功を示す場合は{@code true}を取得する。
     *          処理結果が成功を示さない場合は{code false}を取得する。
     */
    public static boolean isSuccess(Intent intent) {

        if (intent == null) {
            return false;
        }

        if (intent.hasExtra(MposIntegrationKey.STATUS)) {
            return STATUS_SUCCESS.equalsIgnoreCase(intent.getStringExtra(MposIntegrationKey.STATUS));

        } else {
            String packageName = intent.getStringExtra(MposIntegrationKey.CALLBACK_SUCCESS_CLASS_NAME);
            return packageName != null && !packageName.isEmpty();
        }
    }

    /**
     * mPOSアプリからレスポンスとして連携された処理結果が失敗を示すものであるかを判定する。
     *
     * @param intent {@link Intent}
     * @return 処理結果が失敗を示す場合は{@code true}を取得する。
     *          処理結果が失敗を示さない場合は{code false}を取得する。
     */
    public static boolean isError(Intent intent) {

        if (intent == null) {
            return false;
        }

        if (intent.hasExtra(MposIntegrationKey.STATUS)) {
            return STATUS_FAILURE.equalsIgnoreCase(intent.getStringExtra(MposIntegrationKey.STATUS));

        } else {
            String packageName = intent.getStringExtra(MposIntegrationKey.CALLBACK_ERROR_CLASS_NAME);
            return packageName != null && !packageName.isEmpty();
        }
    }

    /**
     * mPOSアプリからレスポンスとして連携された処理結果が中止を示すものであるかを判定する。
     *
     * @param intent {@link Intent}
     * @return 処理結果が中止を示す場合は{@code true}を取得する。
     *          処理結果が中止を示さない場合は{code false}を取得する。
     */
    public static boolean isCancel(Intent intent) {

        if (intent == null) {
            return false;
        }

        if (intent.hasExtra(MposIntegrationKey.STATUS)) {
            return STATUS_CANCEL.equalsIgnoreCase(intent.getStringExtra(MposIntegrationKey.STATUS));

        } else {
            String packageName = intent.getStringExtra(MposIntegrationKey.CALLBACK_CANCEL_CLASS_NAME);
            return packageName != null && !packageName.isEmpty();
        }
    }
}
